@@ -1,107 +1,93 @@
 import uuid
 from itertools import tee
-from typing import Any, Iterable, Optional, Sequence, Union, get_args
+from typing import Any, Iterable, Sequence, get_args
 from copy import deepcopy
-from pathlib import Path
 
 import numpy as np
-
 from pydantic import BaseModel
 
+from qdrant_client import grpc
+from qdrant_client.common.client_warnings import show_warning, show_warning_once
 from qdrant_client.client_base import QdrantBase
+from qdrant_client.embed.embedder import Embedder
+from qdrant_client.embed.model_embedder import ModelEmbedder
+from qdrant_client.http import models
 from qdrant_client.conversions import common_types as types
 from qdrant_client.conversions.conversion import GrpcToRest
 from qdrant_client.embed.common import INFERENCE_OBJECT_TYPES
-from qdrant_client.embed.embed_inspector import InspectorEmbed
-from qdrant_client.embed.models import NumericVector, NumericVectorStruct
 from qdrant_client.embed.schema_parser import ModelSchemaParser
-from qdrant_client.embed.utils import FieldPath
-from qdrant_client.fastembed_common import QueryResponse
-from qdrant_client.http import models
 from qdrant_client.hybrid.fusion import reciprocal_rank_fusion
-from qdrant_client import grpc
-from qdrant_client.common.client_warnings import show_warning
+from qdrant_client.fastembed_common import FastEmbedMisc, OnnxProvider
 
-try:
-    from fastembed import (
-        SparseTextEmbedding,
-        TextEmbedding,
-        LateInteractionTextEmbedding,
-        ImageEmbedding,
-    )
-    from fastembed.common import OnnxProvider
-    from PIL import Image as PilImage
-except ImportError:
-    TextEmbedding = None
-    SparseTextEmbedding = None
-    OnnxProvider = None
-    LateInteractionTextEmbedding = None
-    ImageEmbedding = None
-    PilImage = None
-
-
-SUPPORTED_EMBEDDING_MODELS: dict[str, tuple[int, models.Distance]] = (
-    {
-        model["model"]: (model["dim"], models.Distance.COSINE)
-        for model in TextEmbedding.list_supported_models()
-    }
-    if TextEmbedding
-    else {}
-)
-
-SUPPORTED_SPARSE_EMBEDDING_MODELS: dict[str, tuple[int, models.Distance]] = (
-    {model["model"]: model for model in SparseTextEmbedding.list_supported_models()}
-    if SparseTextEmbedding
-    else {}
-)
-
-IDF_EMBEDDING_MODELS: set[str] = (
-    {
-        model_config["model"]
-        for model_config in SparseTextEmbedding.list_supported_models()
-        if model_config.get("requires_idf", None)
-    }
-    if SparseTextEmbedding
-    else set()
-)
-
-_LATE_INTERACTION_EMBEDDING_MODELS: dict[str, tuple[int, models.Distance]] = (
-    {model["model"]: model for model in LateInteractionTextEmbedding.list_supported_models()}
-    if LateInteractionTextEmbedding
-    else {}
-)
-
-_IMAGE_EMBEDDING_MODELS: dict[str, tuple[int, models.Distance]] = (
-    {model["model"]: model for model in ImageEmbedding.list_supported_models()}
-    if ImageEmbedding
-    else {}
+# region imports used in deprecated methods
+from qdrant_client.fastembed_common import (
+    QueryResponse,
+    TextEmbedding,
+    SparseTextEmbedding,
+    IDF_EMBEDDING_MODELS,
 )
+# endregion
 
 
 class QdrantFastembedMixin(QdrantBase):
     DEFAULT_EMBEDDING_MODEL = "BAAI/bge-small-en"
-
-    embedding_models: dict[str, "TextEmbedding"] = {}
-    sparse_embedding_models: dict[str, "SparseTextEmbedding"] = {}
-    late_interaction_embedding_models: dict[str, "LateInteractionTextEmbedding"] = {}
-    image_embedding_models: dict[str, "ImageEmbedding"] = {}
+    DEFAULT_BATCH_SIZE = 8
     _FASTEMBED_INSTALLED: bool
 
-    def __init__(self, parser: ModelSchemaParser, **kwargs: Any):
-        self._embedding_model_name: Optional[str] = None
-        self._sparse_embedding_model_name: Optional[str] = None
-        self._embed_inspector = InspectorEmbed(parser=parser)
-        try:
-            from fastembed import SparseTextEmbedding, TextEmbedding
+    def __init__(self, parser: ModelSchemaParser, is_local_mode: bool, server_version: str | None):
+        self.__class__._FASTEMBED_INSTALLED = FastEmbedMisc.is_installed()
+        self._embedding_model_name: str | None = None
+        self._sparse_embedding_model_name: str | None = None
 
-            assert len(SparseTextEmbedding.list_supported_models()) > 0
-            assert len(TextEmbedding.list_supported_models()) > 0
+        self._model_embedder = ModelEmbedder(
+            parser=parser, is_local_mode=is_local_mode, server_version=server_version
+        )
+        super().__init__()
 
-            self.__class__._FASTEMBED_INSTALLED = True
-        except ImportError:
-            self.__class__._FASTEMBED_INSTALLED = False
+    @classmethod
+    def list_text_models(cls) -> dict[str, tuple[int, models.Distance]]:
+        """Lists the supported dense text models.
 
-        super().__init__(**kwargs)
+        Returns:
+            dict[str, tuple[int, models.Distance]]: A dict of model names, their dimensions and distance metrics.
+        """
+        return FastEmbedMisc.list_text_models()
+
+    @classmethod
+    def list_image_models(cls) -> dict[str, tuple[int, models.Distance]]:
+        """Lists the supported image dense models.
+
+        Returns:
+            dict[str, tuple[int, models.Distance]]: A dict of model names, their dimensions and distance metrics.
+        """
+        return FastEmbedMisc.list_image_models()
+
+    @classmethod
+    def list_late_interaction_text_models(cls) -> dict[str, tuple[int, models.Distance]]:
+        """Lists the supported late interaction text models.
+
+        Returns:
+            dict[str, tuple[int, models.Distance]]: A dict of model names, their dimensions and distance metrics.
+        """
+        return FastEmbedMisc.list_late_interaction_text_models()
+
+    @classmethod
+    def list_late_interaction_multimodal_models(cls) -> dict[str, tuple[int, models.Distance]]:
+        """Lists the supported late interaction multimodal models.
+
+        Returns:
+            dict[str, tuple[int, models.Distance]]: A dict of model names, their dimensions and distance metrics.
+        """
+        return FastEmbedMisc.list_late_interaction_multimodal_models()
+
+    @classmethod
+    def list_sparse_models(cls) -> dict[str, dict[str, Any]]:
+        """Lists the supported sparse text models.
+
+        Returns:
+            dict[str, dict[str, Any]]: A dict of model names and their descriptions.
+        """
+        return FastEmbedMisc.list_sparse_models()
 
     @property
     def embedding_model_name(self) -> str:
@@ -110,18 +96,18 @@ class QdrantFastembedMixin(QdrantBase):
         return self._embedding_model_name
 
     @property
-    def sparse_embedding_model_name(self) -> Optional[str]:
+    def sparse_embedding_model_name(self) -> str | None:
         return self._sparse_embedding_model_name
 
     def set_model(
         self,
         embedding_model_name: str,
-        max_length: Optional[int] = None,
-        cache_dir: Optional[str] = None,
-        threads: Optional[int] = None,
-        providers: Optional[Sequence["OnnxProvider"]] = None,
+        max_length: int | None = None,
+        cache_dir: str | None = None,
+        threads: int | None = None,
+        providers: Sequence["OnnxProvider"] | None = None,
         cuda: bool = False,
-        device_ids: Optional[list[int]] = None,
+        device_ids: list[int] | None = None,
         lazy_load: bool = False,
         **kwargs: Any,
     ) -> None:
@@ -168,18 +154,19 @@ class QdrantFastembedMixin(QdrantBase):
             cuda=cuda,
             device_ids=device_ids,
             lazy_load=lazy_load,
+            deprecated=True,
             **kwargs,
         )
         self._embedding_model_name = embedding_model_name
 
     def set_sparse_model(
         self,
-        embedding_model_name: Optional[str],
-        cache_dir: Optional[str] = None,
-        threads: Optional[int] = None,
-        providers: Optional[Sequence["OnnxProvider"]] = None,
+        embedding_model_name: str | None,
+        cache_dir: str | None = None,
+        threads: int | None = None,
+        providers: Sequence["OnnxProvider"] | None = None,
         cuda: bool = False,
-        device_ids: Optional[list[int]] = None,
+        device_ids: list[int] | None = None,
         lazy_load: bool = False,
         **kwargs: Any,
     ) -> None:
@@ -218,143 +205,71 @@ class QdrantFastembedMixin(QdrantBase):
                 cuda=cuda,
                 device_ids=device_ids,
                 lazy_load=lazy_load,
+                deprecated=True,
                 **kwargs,
             )
         self._sparse_embedding_model_name = embedding_model_name
 
-    @classmethod
-    def _import_fastembed(cls) -> None:
-        if cls._FASTEMBED_INSTALLED:
-            return
-
-        # If it's not, ask the user to install it
-        raise ImportError(
-            "fastembed is not installed."
-            " Please install it to enable fast vector indexing with `pip install fastembed`."
-        )
-
     @classmethod
     def _get_model_params(cls, model_name: str) -> tuple[int, models.Distance]:
-        cls._import_fastembed()
+        FastEmbedMisc.import_fastembed()
 
-        if model_name not in SUPPORTED_EMBEDDING_MODELS:
+        for descriptions in (
+            FastEmbedMisc.list_text_models(),
+            FastEmbedMisc.list_image_models(),
+            FastEmbedMisc.list_late_interaction_text_models(),
+            FastEmbedMisc.list_late_interaction_multimodal_models(),
+        ):
+            if params := descriptions.get(model_name):
+                return params
+
+        if model_name in FastEmbedMisc.list_sparse_models():
             raise ValueError(
-                f"Unsupported embedding model: {model_name}. Supported models: {SUPPORTED_EMBEDDING_MODELS}"
+                "Sparse embeddings do not return fixed embedding size and distance type"
             )
 
-        return SUPPORTED_EMBEDDING_MODELS[model_name]
+        raise ValueError(f"Unsupported embedding model: {model_name}")
 
-    @classmethod
     def _get_or_init_model(
-        cls,
+        self,
         model_name: str,
-        cache_dir: Optional[str] = None,
-        threads: Optional[int] = None,
-        providers: Optional[Sequence["OnnxProvider"]] = None,
+        cache_dir: str | None = None,
+        threads: int | None = None,
+        providers: Sequence["OnnxProvider"] | None = None,
+        deprecated: bool = False,
         **kwargs: Any,
     ) -> "TextEmbedding":
-        if model_name in cls.embedding_models:
-            return cls.embedding_models[model_name]
+        FastEmbedMisc.import_fastembed()
 
-        cls._import_fastembed()
-
-        if model_name not in SUPPORTED_EMBEDDING_MODELS:
-            raise ValueError(
-                f"Unsupported embedding model: {model_name}. Supported models: {SUPPORTED_EMBEDDING_MODELS}"
-            )
-
-        cls.embedding_models[model_name] = TextEmbedding(
+        assert isinstance(self._model_embedder.embedder, Embedder)
+        return self._model_embedder.embedder.get_or_init_model(
             model_name=model_name,
             cache_dir=cache_dir,
             threads=threads,
             providers=providers,
+            deprecated=deprecated,
             **kwargs,
         )
-        return cls.embedding_models[model_name]
 
-    @classmethod
     def _get_or_init_sparse_model(
-        cls,
+        self,
         model_name: str,
-        cache_dir: Optional[str] = None,
-        threads: Optional[int] = None,
-        providers: Optional[Sequence["OnnxProvider"]] = None,
+        cache_dir: str | None = None,
+        threads: int | None = None,
+        providers: Sequence["OnnxProvider"] | None = None,
+        deprecated: bool = False,
         **kwargs: Any,
     ) -> "SparseTextEmbedding":
-        if model_name in cls.sparse_embedding_models:
-            return cls.sparse_embedding_models[model_name]
-
-        cls._import_fastembed()
-
-        if model_name not in SUPPORTED_SPARSE_EMBEDDING_MODELS:
-            raise ValueError(
-                f"Unsupported embedding model: {model_name}. Supported models: {SUPPORTED_SPARSE_EMBEDDING_MODELS}"
-            )
-
-        cls.sparse_embedding_models[model_name] = SparseTextEmbedding(
+        FastEmbedMisc.import_fastembed()
+        assert isinstance(self._model_embedder.embedder, Embedder)
+        return self._model_embedder.embedder.get_or_init_sparse_model(
             model_name=model_name,
             cache_dir=cache_dir,
             threads=threads,
             providers=providers,
+            deprecated=deprecated,
             **kwargs,
         )
-        return cls.sparse_embedding_models[model_name]
-
-    @classmethod
-    def _get_or_init_late_interaction_model(
-        cls,
-        model_name: str,
-        cache_dir: Optional[str] = None,
-        threads: Optional[int] = None,
-        providers: Optional[Sequence["OnnxProvider"]] = None,
-        **kwargs: Any,
-    ) -> "LateInteractionTextEmbedding":
-        if model_name in cls.late_interaction_embedding_models:
-            return cls.late_interaction_embedding_models[model_name]
-
-        cls._import_fastembed()
-
-        if model_name not in _LATE_INTERACTION_EMBEDDING_MODELS:
-            raise ValueError(
-                f"Unsupported embedding model: {model_name}. Supported models: {_LATE_INTERACTION_EMBEDDING_MODELS}"
-            )
-
-        cls.late_interaction_embedding_models[model_name] = LateInteractionTextEmbedding(
-            model_name=model_name,
-            cache_dir=cache_dir,
-            threads=threads,
-            providers=providers,
-            **kwargs,
-        )
-        return cls.late_interaction_embedding_models[model_name]
-
-    @classmethod
-    def _get_or_init_image_model(
-        cls,
-        model_name: str,
-        cache_dir: Optional[str] = None,
-        threads: Optional[int] = None,
-        providers: Optional[Sequence["OnnxProvider"]] = None,
-        **kwargs: Any,
-    ) -> "ImageEmbedding":
-        if model_name in cls.image_embedding_models:
-            return cls.image_embedding_models[model_name]
-
-        cls._import_fastembed()
-
-        if model_name not in _IMAGE_EMBEDDING_MODELS:
-            raise ValueError(
-                f"Unsupported embedding model: {model_name}. Supported models: {_IMAGE_EMBEDDING_MODELS}"
-            )
-
-        cls.image_embedding_models[model_name] = ImageEmbedding(
-            model_name=model_name,
-            cache_dir=cache_dir,
-            threads=threads,
-            providers=providers,
-            **kwargs,
-        )
-        return cls.image_embedding_models[model_name]
 
     def _embed_documents(
         self,
@@ -362,9 +277,9 @@ class QdrantFastembedMixin(QdrantBase):
         embedding_model_name: str = DEFAULT_EMBEDDING_MODEL,
         batch_size: int = 32,
         embed_type: str = "default",
-        parallel: Optional[int] = None,
+        parallel: int | None = None,
     ) -> Iterable[tuple[str, list[float]]]:
-        embedding_model = self._get_or_init_model(model_name=embedding_model_name)
+        embedding_model = self._get_or_init_model(model_name=embedding_model_name, deprecated=True)
         documents_a, documents_b = tee(documents, 2)
         if embed_type == "passage":
             vectors_iter = embedding_model.passage_embed(
@@ -389,9 +304,11 @@ class QdrantFastembedMixin(QdrantBase):
         documents: Iterable[str],
         embedding_model_name: str = DEFAULT_EMBEDDING_MODEL,
         batch_size: int = 32,
-        parallel: Optional[int] = None,
+        parallel: int | None = None,
     ) -> Iterable[types.SparseVector]:
-        sparse_embedding_model = self._get_or_init_sparse_model(model_name=embedding_model_name)
+        sparse_embedding_model = self._get_or_init_sparse_model(
+            model_name=embedding_model_name, deprecated=True
+        )
 
         vectors_iter = sparse_embedding_model.embed(
             documents, batch_size=batch_size, parallel=parallel
@@ -412,7 +329,7 @@ class QdrantFastembedMixin(QdrantBase):
         model_name = self.embedding_model_name.split("/")[-1].lower()
         return f"fast-{model_name}"
 
-    def get_sparse_vector_field_name(self) -> Optional[str]:
+    def get_sparse_vector_field_name(self) -> str | None:
         """
         Returns name of the vector field in qdrant collection, used by current fastembed model.
         Returns:
@@ -459,11 +376,11 @@ class QdrantFastembedMixin(QdrantBase):
 
     def _points_iterator(
         self,
-        ids: Optional[Iterable[models.ExtendedPointId]],
-        metadata: Optional[Iterable[dict[str, Any]]],
+        ids: Iterable[models.ExtendedPointId] | None,
+        metadata: Iterable[dict[str, Any]] | None,
         encoded_docs: Iterable[tuple[str, list[float]]],
         ids_accumulator: list,
-        sparse_vectors: Optional[Iterable[types.SparseVector]] = None,
+        sparse_vectors: Iterable[types.SparseVector] | None = None,
     ) -> Iterable[models.PointStruct]:
         if ids is None:
             ids = iter(lambda: uuid.uuid4().hex, None)
@@ -523,11 +440,31 @@ class QdrantFastembedMixin(QdrantBase):
                     modifier == models.Modifier.IDF
                 ), f"{self.sparse_embedding_model_name} requires modifier IDF, current modifier is {modifier}"
 
+    def get_embedding_size(
+        self,
+        model_name: str | None = None,
+    ) -> int:
+        """Get the size of the embeddings produced by the specified model.
+
+        Args:
+            model_name: optional, the name of the model to get the embedding size for. If None, the default model will
+                be used.
+
+        Returns:
+            int: the size of the embeddings produced by the model.
+
+        Raises:
+            ValueError: If sparse model name is passed or model is not found in the supported models.
+        """
+        model_name = model_name or self.embedding_model_name
+        embeddings_size, _ = self._get_model_params(model_name=model_name)
+        return embeddings_size
+
     def get_fastembed_vector_params(
         self,
-        on_disk: Optional[bool] = None,
-        quantization_config: Optional[models.QuantizationConfig] = None,
-        hnsw_config: Optional[models.HnswConfigDiff] = None,
+        on_disk: bool | None = None,
+        quantization_config: models.QuantizationConfig | None = None,
+        hnsw_config: models.HnswConfigDiff | None = None,
     ) -> dict[str, models.VectorParams]:
         """
         Generates vector configuration, compatible with fastembed models.
@@ -554,9 +491,9 @@ class QdrantFastembedMixin(QdrantBase):
 
     def get_fastembed_sparse_vector_params(
         self,
-        on_disk: Optional[bool] = None,
-        modifier: Optional[models.Modifier] = None,
-    ) -> Optional[dict[str, models.SparseVectorParams]]:
+        on_disk: bool | None = None,
+        modifier: models.Modifier | None = None,
+    ) -> dict[str, models.SparseVectorParams] | None:
         """
         Generates vector configuration, compatible with fastembed sparse models.
 
@@ -586,12 +523,12 @@ class QdrantFastembedMixin(QdrantBase):
         self,
         collection_name: str,
         documents: Iterable[str],
-        metadata: Optional[Iterable[dict[str, Any]]] = None,
-        ids: Optional[Iterable[models.ExtendedPointId]] = None,
+        metadata: Iterable[dict[str, Any]] | None = None,
+        ids: Iterable[models.ExtendedPointId] | None = None,
         batch_size: int = 32,
-        parallel: Optional[int] = None,
+        parallel: int | None = None,
         **kwargs: Any,
-    ) -> list[Union[str, int]]:
+    ) -> list[str | int]:
         """
         Adds text documents into qdrant collection.
         If collection does not exist, it will be created with default parameters.
@@ -623,6 +560,11 @@ class QdrantFastembedMixin(QdrantBase):
             List of IDs of added documents. If no ids provided, UUIDs will be randomly generated on client side.
 
         """
+        show_warning_once(
+            "`add` method has been deprecated and will be removed in 1.17. "
+            "Instead, inference can be done internally within regular methods like `upsert` by wrapping "
+            "data into `models.Document` or `models.Image`."
+        )
 
         # check if we have fastembed installed
         encoded_docs = self._embed_documents(
@@ -680,7 +622,7 @@ class QdrantFastembedMixin(QdrantBase):
         self,
         collection_name: str,
         query_text: str,
-        query_filter: Optional[models.Filter] = None,
+        query_filter: models.Filter | None = None,
         limit: int = 10,
         **kwargs: Any,
     ) -> list[QueryResponse]:
@@ -698,33 +640,38 @@ class QdrantFastembedMixin(QdrantBase):
                 - Exclude vectors which doesn't fit given conditions.
                 - If `None` - search among all vectors
             limit: How many results return
-            **kwargs: Additional search parameters. See `qdrant_client.models.SearchRequest` for details.
+            **kwargs: Additional search parameters. See `qdrant_client.models.QueryRequest` for details.
 
         Returns:
             list[types.ScoredPoint]: List of scored points.
 
         """
-
-        embedding_model_inst = self._get_or_init_model(model_name=self.embedding_model_name)
+        show_warning_once(
+            "`query` method has been deprecated and will be removed in 1.17. "
+            "Instead, inference can be done internally within regular methods like `query_points` by wrapping "
+            "data into `models.Document` or `models.Image`."
+        )
+        embedding_model_inst = self._get_or_init_model(
+            model_name=self.embedding_model_name, deprecated=True
+        )
         embeddings = list(embedding_model_inst.query_embed(query=query_text))
         query_vector = embeddings[0].tolist()
 
         if self.sparse_embedding_model_name is None:
             return self._scored_points_to_query_responses(
-                self.search(
+                self.query_points(
                     collection_name=collection_name,
-                    query_vector=models.NamedVector(
-                        name=self.get_vector_field_name(), vector=query_vector
-                    ),
+                    query=query_vector,
+                    using=self.get_vector_field_name(),
                     query_filter=query_filter,
                     limit=limit,
                     with_payload=True,
                     **kwargs,
-                )
+                ).points
             )
 
         sparse_embedding_model_inst = self._get_or_init_sparse_model(
-            model_name=self.sparse_embedding_model_name
+            model_name=self.sparse_embedding_model_name, deprecated=True
         )
         sparse_vector = list(sparse_embedding_model_inst.query_embed(query=query_text))[0]
         sparse_query_vector = models.SparseVector(
@@ -732,39 +679,37 @@ class QdrantFastembedMixin(QdrantBase):
             values=sparse_vector.values.tolist(),
         )
 
-        dense_request = models.SearchRequest(
-            vector=models.NamedVector(
-                name=self.get_vector_field_name(),
-                vector=query_vector,
-            ),
+        dense_request = models.QueryRequest(
+            query=query_vector,
+            using=self.get_vector_field_name(),
             filter=query_filter,
             limit=limit,
             with_payload=True,
             **kwargs,
         )
-        sparse_request = models.SearchRequest(
-            vector=models.NamedSparseVector(
-                name=self.get_sparse_vector_field_name(),
-                vector=sparse_query_vector,
-            ),
+        sparse_request = models.QueryRequest(
+            query=sparse_query_vector,
+            using=self.get_sparse_vector_field_name(),
             filter=query_filter,
             limit=limit,
             with_payload=True,
             **kwargs,
         )
 
-        dense_request_response, sparse_request_response = self.search_batch(
+        dense_request_response, sparse_request_response = self.query_batch_points(
             collection_name=collection_name, requests=[dense_request, sparse_request]
         )
         return self._scored_points_to_query_responses(
-            reciprocal_rank_fusion([dense_request_response, sparse_request_response], limit=limit)
+            reciprocal_rank_fusion(
+                [dense_request_response.points, sparse_request_response.points], limit=limit
+            )
         )
 
     def query_batch(
         self,
         collection_name: str,
         query_texts: list[str],
-        query_filter: Optional[models.Filter] = None,
+        query_filter: models.Filter | None = None,
         limit: int = 10,
         **kwargs: Any,
     ) -> list[list[QueryResponse]]:
@@ -782,20 +727,26 @@ class QdrantFastembedMixin(QdrantBase):
                 - If `None` - search among all vectors
                 This filter will be applied to all search requests.
             limit: How many results return
-            **kwargs: Additional search parameters. See `qdrant_client.models.SearchRequest` for details.
+            **kwargs: Additional search parameters. See `qdrant_client.models.QueryRequest` for details.
 
         Returns:
             list[list[QueryResponse]]: List of lists of responses for each query text.
 
         """
-        embedding_model_inst = self._get_or_init_model(model_name=self.embedding_model_name)
+        show_warning_once(
+            "`query_batch` method has been deprecated and will be removed in 1.17. "
+            "Instead, inference can be done internally within regular methods like `query_batch_points` by wrapping "
+            "data into `models.Document` or `models.Image`."
+        )
+        embedding_model_inst = self._get_or_init_model(
+            model_name=self.embedding_model_name, deprecated=True
+        )
         query_vectors = list(embedding_model_inst.query_embed(query=query_texts))
         requests = []
         for vector in query_vectors:
-            request = models.SearchRequest(
-                vector=models.NamedVector(
-                    name=self.get_vector_field_name(), vector=vector.tolist()
-                ),
+            request = models.QueryRequest(
+                query=vector.tolist(),
+                using=self.get_vector_field_name(),
                 filter=query_filter,
                 limit=limit,
                 with_payload=True,
@@ -805,14 +756,16 @@ class QdrantFastembedMixin(QdrantBase):
             requests.append(request)
 
         if self.sparse_embedding_model_name is None:
-            responses = self.search_batch(
+            responses = self.query_batch_points(
                 collection_name=collection_name,
                 requests=requests,
             )
-            return [self._scored_points_to_query_responses(response) for response in responses]
+            return [
+                self._scored_points_to_query_responses(response.points) for response in responses
+            ]
 
         sparse_embedding_model_inst = self._get_or_init_sparse_model(
-            model_name=self.sparse_embedding_model_name
+            model_name=self.sparse_embedding_model_name, deprecated=True
         )
         sparse_query_vectors = [
             models.SparseVector(
@@ -822,11 +775,9 @@ class QdrantFastembedMixin(QdrantBase):
             for sparse_vector in sparse_embedding_model_inst.embed(documents=query_texts)
         ]
         for sparse_vector in sparse_query_vectors:
-            request = models.SearchRequest(
-                vector=models.NamedSparseVector(
-                    name=self.get_sparse_vector_field_name(),
-                    vector=sparse_vector,
-                ),
+            request = models.QueryRequest(
+                using=self.get_sparse_vector_field_name(),
+                query=sparse_vector,
                 filter=query_filter,
                 limit=limit,
                 with_payload=True,
@@ -835,7 +786,7 @@ class QdrantFastembedMixin(QdrantBase):
 
             requests.append(request)
 
-        responses = self.search_batch(
+        responses = self.query_batch_points(
             collection_name=collection_name,
             requests=requests,
         )
@@ -843,7 +794,7 @@ class QdrantFastembedMixin(QdrantBase):
         dense_responses = responses[: len(query_texts)]
         sparse_responses = responses[len(query_texts) :]
         responses = [
-            reciprocal_rank_fusion([dense_response, sparse_response], limit=limit)
+            reciprocal_rank_fusion([dense_response.points, sparse_response.points], limit=limit)
             for dense_response, sparse_response in zip(dense_responses, sparse_responses)
         ]
 
@@ -852,19 +803,17 @@ class QdrantFastembedMixin(QdrantBase):
     @classmethod
     def _resolve_query(
         cls,
-        query: Union[
-            types.PointId,
-            list[float],
-            list[list[float]],
-            types.SparseVector,
-            types.Query,
-            types.NumpyArray,
-            models.Document,
-            models.Image,
-            models.InferenceObject,
-            None,
-        ],
-    ) -> Optional[models.Query]:
+        query: types.PointId
+        | list[float]
+        | list[list[float]]
+        | types.SparseVector
+        | types.Query
+        | types.NumpyArray
+        | models.Document
+        | models.Image
+        | models.InferenceObject
+        | None,
+    ) -> models.Query | None:
         """Resolves query interface into a models.Query object
 
         Args:
@@ -876,7 +825,7 @@ class QdrantFastembedMixin(QdrantBase):
         Raises:
             ValueError: if query is not of supported type
         """
-        if isinstance(query, get_args(types.Query)) or isinstance(query, grpc.Query):
+        if isinstance(query, get_args(types.Query)):
             return query
 
         if isinstance(query, types.SparseVector):
@@ -893,7 +842,7 @@ class QdrantFastembedMixin(QdrantBase):
             )
             return models.NearestQuery(nearest=query)
 
-        if isinstance(query, INFERENCE_OBJECT_TYPES):
+        if isinstance(query, get_args(INFERENCE_OBJECT_TYPES)):
             return models.NearestQuery(nearest=query)
 
         if query is None:
@@ -929,178 +878,24 @@ class QdrantFastembedMixin(QdrantBase):
 
     def _embed_models(
         self,
-        model: BaseModel,
-        paths: Optional[list[FieldPath]] = None,
+        raw_models: BaseModel | Iterable[BaseModel],
         is_query: bool = False,
-    ) -> Union[BaseModel, NumericVector]:
-        """Embed model's fields requiring inference
+        batch_size: int | None = None,
+    ) -> Iterable[BaseModel]:
+        yield from self._model_embedder.embed_models(
+            raw_models=raw_models,
+            is_query=is_query,
+            batch_size=batch_size or self.DEFAULT_BATCH_SIZE,
+        )
 
-        Args:
-            model: Qdrant http model containing fields to embed
-            paths: Path to fields to embed. E.g. [FieldPath(current="recommend", tail=[FieldPath(current="negative", tail=None)])]
-            is_query: Flag to determine which embed method to use. Defaults to False.
-
-        Returns:
-            A deepcopy of the method with embedded fields
-        """
-        if paths is None:
-            if isinstance(model, INFERENCE_OBJECT_TYPES):
-                return self._embed_raw_data(model, is_query=is_query)
-            model = deepcopy(model)
-            paths = self._embed_inspector.inspect(model)
-        for path in paths:
-            list_model = [model] if not isinstance(model, list) else model
-            for item in list_model:
-                current_model = getattr(item, path.current, None)
-                if current_model is None:
-                    continue
-                if path.tail:
-                    self._embed_models(current_model, path.tail, is_query=is_query)
-                else:
-                    was_list = isinstance(current_model, list)
-                    current_model = (
-                        [current_model] if not isinstance(current_model, list) else current_model
-                    )
-                    embeddings = [
-                        self._embed_raw_data(data, is_query=is_query) for data in current_model
-                    ]
-                    if was_list:
-                        setattr(item, path.current, embeddings)
-                    else:
-                        setattr(item, path.current, embeddings[0])
-        return model
-
-    @staticmethod
-    def _resolve_inference_object(data: models.VectorStruct) -> models.VectorStruct:
-        """Resolve inference object into a model
-
-        Args:
-            data: models.VectorStruct - data to resolve, if it's an inference object, convert it to a proper type,
-                otherwise - keep unchanged
-
-        Returns:
-            models.VectorStruct: resolved data
-        """
-
-        if not isinstance(data, models.InferenceObject):
-            return data
-
-        model_name = data.model
-        value = data.object
-        options = data.options
-        if model_name in (
-            *SUPPORTED_EMBEDDING_MODELS.keys(),
-            *SUPPORTED_SPARSE_EMBEDDING_MODELS.keys(),
-            *_LATE_INTERACTION_EMBEDDING_MODELS.keys(),
-        ):
-            return models.Document(model=model_name, text=value, options=options)
-        if model_name in _IMAGE_EMBEDDING_MODELS:
-            return models.Image(model=model_name, image=value, options=options)
-
-        raise ValueError(f"{model_name} is not among supported models")
-
-    def _embed_raw_data(
+    def _embed_models_strict(
         self,
-        data: models.VectorStruct,
-        is_query: bool = False,
-    ) -> NumericVectorStruct:
-        """Iterates over the data and calls inference on the fields requiring it
-
-        Args:
-            data: models.VectorStruct - data to embed, if it's not a field which requires inference, leave it as is
-            is_query: Flag to determine which embed method to use. Defaults to False.
-
-        Returns:
-            NumericVectorStruct: Embedded data
-        """
-        data = self._resolve_inference_object(data)
-
-        if isinstance(data, models.Document):
-            return self._embed_document(data, is_query=is_query)
-        elif isinstance(data, models.Image):
-            return self._embed_image(data)
-        elif isinstance(data, dict):
-            return {
-                key: self._embed_raw_data(value, is_query=is_query) for key, value in data.items()
-            }
-        elif isinstance(data, list):
-            # we don't want to iterate over a vector
-            if data and isinstance(data[0], float):
-                return data
-            return [self._embed_raw_data(value, is_query=is_query) for value in data]
-        return data
-
-    def _embed_document(self, document: models.Document, is_query: bool = False) -> NumericVector:
-        """Embed a document using the specified embedding model
-
-        Args:
-            document: Document to embed
-            is_query: Flag to determine which embed method to use. Defaults to False.
-
-        Returns:
-            NumericVector: Document's embedding
-
-        Raises:
-            ValueError: If model is not supported
-        """
-        model_name = document.model
-        text = document.text
-        options = document.options or {}
-        if model_name in SUPPORTED_EMBEDDING_MODELS:
-            embedding_model_inst = self._get_or_init_model(model_name=model_name, **options)
-            if not is_query:
-                embedding = list(embedding_model_inst.embed(documents=[text]))[0].tolist()
-            else:
-                embedding = list(embedding_model_inst.query_embed(query=text))[0].tolist()
-            return embedding
-        elif model_name in SUPPORTED_SPARSE_EMBEDDING_MODELS:
-            sparse_embedding_model_inst = self._get_or_init_sparse_model(
-                model_name=model_name, **options
-            )
-            if not is_query:
-                sparse_embedding = list(sparse_embedding_model_inst.embed(documents=[text]))[0]
-            else:
-                sparse_embedding = list(sparse_embedding_model_inst.query_embed(query=text))[0]
-
-            return models.SparseVector(
-                indices=sparse_embedding.indices.tolist(), values=sparse_embedding.values.tolist()
-            )
-        elif model_name in _LATE_INTERACTION_EMBEDDING_MODELS:
-            li_embedding_model_inst = self._get_or_init_late_interaction_model(
-                model_name=model_name, **options
-            )
-            if not is_query:
-                embedding = list(li_embedding_model_inst.embed(documents=[text]))[0].tolist()
-            else:
-                embedding = list(li_embedding_model_inst.query_embed(query=text))[0].tolist()
-            return embedding
-        else:
-            raise ValueError(f"{model_name} is not among supported models")
-
-    def _embed_image(self, image: models.Image) -> NumericVector:
-        """Embed an image using the specified embedding model
-
-        Args:
-            image: Image to embed
-
-        Returns:
-            NumericVector: Image's embedding
-
-        Raises:
-            ValueError: If model is not supported
-        """
-        model_name = image.model
-        if model_name in _IMAGE_EMBEDDING_MODELS:
-            embedding_model_inst = self._get_or_init_image_model(
-                model_name=model_name, **(image.options or {})
-            )
-            if not isinstance(image.image, (str, Path, PilImage.Image)):  # type: ignore
-                # PilImage is None if PIL is not installed,
-                # but we'll fail earlier if it's not installed.
-                raise ValueError(
-                    f"Unsupported image type: {type(image.image)}. Image: {image.image}"
-                )
-            embedding = list(embedding_model_inst.embed(images=[image.image]))[0].tolist()
-            return embedding
-
-        raise ValueError(f"{model_name} is not among supported models")
+        raw_models: Iterable[dict[str, BaseModel] | BaseModel],
+        batch_size: int | None = None,
+        parallel: int | None = None,
+    ) -> Iterable[BaseModel]:
+        yield from self._model_embedder.embed_models_strict(
+            raw_models=raw_models,
+            batch_size=batch_size or self.DEFAULT_BATCH_SIZE,
+            parallel=parallel,
+        )
